@@ -1,6 +1,20 @@
-/*
- *
- */
+/*****************************************************************************
+*    Open LiteSpeed is an open source HTTP server.                           *
+*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*                                                                            *
+*    This program is free software: you can redistribute it and/or modify    *
+*    it under the terms of the GNU General Public License as published by    *
+*    the Free Software Foundation, either version 3 of the License, or       *
+*    (at your option) any later version.                                     *
+*                                                                            *
+*    This program is distributed in the hope that it will be useful,         *
+*    but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            *
+*    GNU General Public License for more details.                            *
+*                                                                            *
+*    You should have received a copy of the GNU General Public License       *
+*    along with this program. If not, see http://www.gnu.org/licenses/.      *
+*****************************************************************************/
 #define DEBUG_SHM_OFFSETS 0
 
 #if defined(__APPLE__)
@@ -236,8 +250,10 @@ int UdpListener::start()
     else
         ret = setsockopt(fd, IPPROTO_IPV6, IPV6_RECVPKTINFO, &val, sizeof(val));
     if (ret != 0)
+    {
+        close(fd);
         return -1;
-
+    }
 #if (__linux__ && !defined(IP_RECVORIGDSTADDR)) || __APPLE__
     /* Need to set IP_PKTINFO for sending */
     if (AF_INET == m_addr.get()->sa_family)
