@@ -26,6 +26,11 @@
 #include <util/datetime.h>
 #include <log4cxx/logger.h>
 
+#include <assert.h>
+#if __cplusplus <= 199711L && !defined(static_assert)
+#define static_assert(a, b) _Static_assert(a, b)
+#endif
+
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/pem.h>
@@ -48,6 +53,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <time.h>
 #include <unistd.h>
 
 static char * s_pOcspCachePath = NULL;
@@ -109,10 +115,14 @@ int SslOcspStapling::callback(SSL *ssl)
     iResult = SSL_TLSEXT_ERR_NOACK;
     if (m_RespTime == UINT_MAX)
         return iResult;
+    unsigned char *resp_data = m_pRespData;
     update();
     if (m_pRespData && m_iDataLen > 0)
     {
-#ifndef OPENSSL_IS_BORINGSSL
+#ifdef OPENSSL_IS_BORINGSSL
+        if (m_pRespData != resp_data)
+            SSL_set_ocsp_response(ssl, m_pRespData, m_iDataLen);
+#else
         unsigned char *pOcspResp;
         /*OpenSSL will free pOcspResp by itself */
         pOcspResp = (unsigned char *)malloc(m_iDataLen);
@@ -136,7 +146,7 @@ int SslOcspStapling::processResponse(HttpFetch *pHttpFetch)
     if (istatusCode == 200 && pRespContentType != NULL
         && strcasecmp(pRespContentType, "application/ocsp-response") == 0)
     {
-        if (verifyRespFile() != 0)
+        if (verifyRespFile(1) != 0)
         {
             m_pCtx->disableOscp();
             m_RespTime = UINT_MAX;
@@ -163,10 +173,13 @@ SslOcspStapling::SslOcspStapling()
     : m_pHttpFetch(NULL)
     , m_pReqData(NULL)
     , m_iDataLen(0)
-    , m_pRespData(NULL)
     , m_iocspRespMaxAge(3600 * 24)
+    , m_pRespData(NULL)
+    , m_notBefore(NULL)
     , m_pCtx(NULL)
     , m_RespTime(0)
+    , m_statTime(0)
+    , m_nextUpdate(0)
     , m_pCertId(NULL)
 {
 }
@@ -180,6 +193,8 @@ SslOcspStapling::~SslOcspStapling()
         free(m_pReqData);
     if (m_pCertId != NULL)
         OCSP_CERTID_free(m_pCertId);
+    if (m_notBefore)
+        ASN1_STRING_free(m_notBefore);
 }
 
 int SslOcspStapling::init(SslContext *pSslCtx)
@@ -206,6 +221,13 @@ int SslOcspStapling::init(SslContext *pSslCtx)
         //                           m_sOcspResponder.len());
         iResult = 0;
         //update();
+        const ASN1_TIME *not_before;
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L || defined(OPENSSL_IS_BORINGSSL)
+        not_before = X509_get0_notBefore(pCert);
+#else
+        not_before = X509_get_notBefore(pCert);
+#endif
+        ASN1_TIME_to_generalizedtime((ASN1_TIME *)not_before, &m_notBefore);
     }
     X509_free(pCert);
     return iResult;
@@ -217,14 +239,30 @@ int SslOcspStapling::update()
     struct stat st;
     if (m_RespTime == UINT_MAX)
         return 0;
-    if (m_RespTime != 0 && m_RespTime + m_iocspRespMaxAge < DateTime::s_curTime)
+    //NOTE: test code , to test clearing out expired OCSP response
+    //if (m_pRespData && m_statTime + 10 <= DateTime::s_curTime)
+    //    m_nextUpdate = DateTime::s_curTime;
+    if (m_pRespData && m_nextUpdate <= DateTime::s_curTime)
     {
-        return 0;
+        LS_DBG("[OCSP] %s: OCSP response expired, cur: %ld, resp_create: %ld, expire: %ld.\n",
+                    m_sRespfile.c_str(), DateTime::s_curTime, m_RespTime, m_nextUpdate);
+        releaseRespData();
+        if (m_statTime != DateTime::s_curTime)
+        {
+            m_statTime = DateTime::s_curTime;
+            goto TRY_CREATE_REQ;
+        }
+        else
+            return 0;
     }
+    if (m_statTime != 0 && m_statTime + 60 >= DateTime::s_curTime)
+        return 0;
+
+    m_statTime = DateTime::s_curTime;
 
     if (::stat(m_sRespfile.c_str(), &st) == 0)
     {
-        if (st.st_mtime + m_iocspRespMaxAge + 3600 < DateTime::s_curTime)
+        if (st.st_mtime + m_iocspRespMaxAge < DateTime::s_curTime)
         {
             releaseRespData();
             unlink(m_sRespfile.c_str());
@@ -233,6 +271,10 @@ int SslOcspStapling::update()
         {
             if (m_RespTime == st.st_mtime)
                 return 0;
+
+            LS_DBG("[OCSP] %s: file timestamp updated, current: %ld, file: %ld\n",
+                      m_sRespfile.c_str(), m_RespTime, st.st_mtime);
+
             if (verifyRespFile(0) == LS_OK)
             {
                 m_RespTime = st.st_mtime;
@@ -240,6 +282,8 @@ int SslOcspStapling::update()
             }
         }
     }
+
+TRY_CREATE_REQ:
     ret = ::stat(m_sRespfileTmp.c_str(), &st);
     if (ret != 0 || st.st_mtime + 30 < DateTime::s_curTime)
     {
@@ -358,7 +402,7 @@ int SslOcspStapling::createRequest()
         {
             if (DateTime::s_curTime - m_pHttpFetch->getTimeStart() < 30)
                 return 0;
-            LS_DBG("%s: HTTP fetch timed out, state: %d\n",
+            LS_DBG("[OCSP] %s: HTTP fetch timed out, state: %d\n",
                       m_sRespfileTmp.c_str(), m_pHttpFetch->getReqState());
         }
         delete m_pHttpFetch;
@@ -401,7 +445,7 @@ void SslOcspStapling::releaseRespData()
 }
 
 
-void SslOcspStapling::updateRespData(OCSP_RESPONSE *pResponse)
+int SslOcspStapling::updateRespData(OCSP_RESPONSE *pResponse)
 {
     unsigned char *pOcspResp;
     m_iDataLen = i2d_OCSP_RESPONSE(pResponse, NULL);
@@ -418,21 +462,35 @@ void SslOcspStapling::updateRespData(OCSP_RESPONSE *pResponse)
         if (m_iDataLen <= 0)
         {
             releaseRespData();
+            return -1;
         }
 #ifdef OPENSSL_IS_BORINGSSL
         else if (m_pCtx)
             SSL_CTX_set_ocsp_response(m_pCtx->get(), m_pRespData, m_iDataLen);
 #endif
+        return 0;
     }
+    return -1;
 }
 
+
+time_t ASN1_TIME_to_time_t(const ASN1_TIME * asn1)
+{
+    int days = 0, seconds = 0;
+    if (!ASN1_TIME_diff(&days, &seconds, NULL, asn1))
+        return -1;
+    time_t t = time(NULL);
+    t += days * 86400 + seconds;
+    return t;
+}
+
+
 int SslOcspStapling::certVerify(OCSP_RESPONSE *pResponse,
                                 OCSP_BASICRESP *pBasicResp, X509_STORE *pXstore)
 {
     int                 n, iResult = -1;
     STACK_OF(X509)      *pXchain;
     ASN1_GENERALIZEDTIME  *pThisupdate, *pNextupdate;
-    struct stat         st;
 
 #ifndef OPENSSL_IS_BORINGSSL
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
@@ -448,38 +506,48 @@ int SslOcspStapling::certVerify(OCSP_RESPONSE *pResponse,
     {
         if (m_pCertId == NULL)
         {
-            log4cxx::Logger::getRootLogger()->error("%s: cert ID is NULL\n",
-                                       m_sCertfile.c_str());
+            LS_NOTICE("[OCSP] %s: cert ID is NULL\n", m_sCertfile.c_str());
             return 1;
         }
         int find_status = OCSP_resp_find_status(pBasicResp, m_pCertId, &n,
                                       NULL, NULL, &pThisupdate, &pNextupdate);
-        //NOTE: get around an issue with response have wrong nextupdate time.
-        //      do not check it for now.
-        pNextupdate = NULL;
-        int validate = OCSP_check_validity(pThisupdate, pNextupdate, 300, -1);
+        int validate = -100;
 
-        if (find_status == 1 && n == V_OCSP_CERTSTATUS_GOOD
-            && validate == 1)
+        if (find_status == 1 && n == V_OCSP_CERTSTATUS_GOOD)
         {
+            int day, sec;
+            //NOTE: get around an issue with response have wrong nextupdate time.
+            //      make sure it is later than thisupdate.
+            if (pThisupdate && pNextupdate)
+            {
+                if (ASN1_TIME_diff(&day, &sec, pThisupdate, pNextupdate) == 0
+                    || (day <= 0 && sec <= 0))
+                {
+                    LS_NOTICE("[OCSP] %s: Bad next_update time day: %d, sec: %d, "
+                                "set it to NULL\n", m_sCertfile.c_str(), day, sec);
+                    pNextupdate = NULL;
+                }
+            }
+            if (m_notBefore && ASN1_TIME_diff(&day, &sec, m_notBefore, pThisupdate)
+                && (day > 0 || sec > 0))
+                validate = OCSP_check_validity(pThisupdate, pNextupdate, 300, -1);
+        }
+
+        if (validate == 1)
+        {
+            m_nextUpdate = ASN1_TIME_to_time_t(pNextupdate);
             iResult = 0;
-            updateRespData(pResponse);
-            unlink(m_sRespfile.c_str());
-            rename(m_sRespfileTmp.c_str(), m_sRespfile.c_str());
-            if (::stat(m_sRespfile.c_str(), &st) == 0)
-                m_RespTime = st.st_mtime;
         }
         else
         {
-            log4cxx::Logger::getRootLogger()->error(
-                "%s: verify failed, find_status: %d, status: %d, validate: %d\n",
-                m_sCertfile.c_str(), find_status, n, validate);
+            LS_NOTICE("[OCSP] %s: verify failed, find_status: %d, status: %d, validate: %d\n",
+                      m_sCertfile.c_str(), find_status, n, validate);
         }
     }
     else
     {
-        log4cxx::Logger::getRootLogger()->error("%s: OCSP_basic_verify() failed: %s\n",
-                                       m_sCertfile.c_str(), SslError().what());
+        LS_NOTICE("[OCSP] %s: OCSP_basic_verify() failed: %s\n",
+                  m_sCertfile.c_str(), SslError().what());
         m_pCtx->disableOscp();
         m_RespTime = UINT_MAX;
 
@@ -495,42 +563,76 @@ int SslOcspStapling::certVerify(OCSP_RESPONSE *pResponse,
     return iResult;
 }
 
-int SslOcspStapling::verifyRespFile(int iNeedVerify)
+int SslOcspStapling::verifyRespFile(int is_new_resp)
 {
     int                 iResult = -1;
     BIO                 *pBio;
     OCSP_RESPONSE       *pResponse;
     OCSP_BASICRESP      *pBasicResp;
+    uint8_t             *pBuf = NULL;
+    const unsigned char *pCopy;
+    size_t              ulSize;
     X509_STORE *pXstore;
-    if (iNeedVerify)
-        pBio = BIO_new_file(m_sRespfileTmp.c_str(), "r");
+    const char *file_name;
+    if (is_new_resp)
+        file_name = m_sRespfileTmp.c_str();
     else
-        pBio = BIO_new_file(m_sRespfile.c_str(), "r");
-    if (pBio == NULL)
-        return -1;
+        file_name = m_sRespfile.c_str();
 
-    pResponse = d2i_OCSP_RESPONSE_bio(pBio, NULL);
-    BIO_free(pBio);
-    if (pResponse == NULL)
+    pBio = BIO_new_file(file_name, "r");
+    if (pBio == NULL)
+    {
+        LS_ERROR("[OCSP] %s: Failed to open file.\n", file_name);
         return -1;
+    }
+
+#ifdef OPENSSL_IS_BORINGSSL
+    if (1 != BIO_read_asn1(pBio, &pBuf, &ulSize, 100 * 1024))
+    {
+        BIO_free(pBio);
+        return -1;
+    }
+    BIO_free(pBio);
+
+    pCopy = pBuf;
+    pResponse = d2i_OCSP_RESPONSE(NULL, &pCopy, ulSize);
+    OPENSSL_free(pBuf);
+#else
+    pResponse = d2i_OCSP_RESPONSE_bio(pBio, NULL);
+#endif
+    if (pResponse == NULL)
+    {
+        LS_ERROR("[OCSP] %s: Failed to load OCSP RESPONSE.\n", file_name);
+        return -1;
+    }
 
     if (OCSP_response_status(pResponse) == OCSP_RESPONSE_STATUS_SUCCESSFUL)
     {
-        if (iNeedVerify)
+        pBasicResp = OCSP_response_get1_basic(pResponse);
+        if (pBasicResp != NULL)
         {
-            pBasicResp = OCSP_response_get1_basic(pResponse);
-            if (pBasicResp != NULL)
+            pXstore = SSL_CTX_get_cert_store(m_pCtx->get());
+            if (pXstore)
             {
-                pXstore = SSL_CTX_get_cert_store(m_pCtx->get());
-                if (pXstore)
-                    iResult = certVerify(pResponse, pBasicResp, pXstore);
-                OCSP_BASICRESP_free(pBasicResp);
+                iResult = certVerify(pResponse, pBasicResp, pXstore);
+                if (iResult == 0 && is_new_resp)
+                {
+                    LS_DBG("[OCSP] %s: Update OCSP response file\n",
+                        m_sRespfile.c_str());
+                    unlink(m_sRespfile.c_str());
+                    rename(m_sRespfileTmp.c_str(), m_sRespfile.c_str());
+                    struct stat st;
+                    if (::stat(m_sRespfile.c_str(), &st) == 0)
+                        m_RespTime = st.st_mtime;
+                }
             }
+            OCSP_BASICRESP_free(pBasicResp);
         }
-        else
+        if (iResult == 0)
         {
-            updateRespData(pResponse);
-            iResult = 0;
+            iResult = updateRespData(pResponse);
+            LS_DBG("[OCSP] %s: Update OCSP response data: %s\n",
+                   file_name, (iResult == 0) ? "succeed" : "fail");
         }
     }
     OCSP_RESPONSE_free(pResponse);
@@ -578,25 +680,25 @@ int SslOcspStapling::getCertId(X509 *pCert)
     pXstore = SSL_CTX_get_cert_store(m_pCtx->get());
     if (pXstore == NULL)
     {
-        setLastErrMsg("SSL_CTX_get_cert_store failed!\n");
+        setLastErrMsg("SSL_CTX_get_cert_store failed!");
         return -1;
     }
     pXstore_ctx = X509_STORE_CTX_new();
     if (pXstore_ctx == NULL)
     {
-        setLastErrMsg("X509_STORE_CTX_new failed!\n");
+        setLastErrMsg("X509_STORE_CTX_new failed!");
         return -1;
     }
     if (X509_STORE_CTX_init(pXstore_ctx, pXstore, NULL, NULL) == 0)
     {
-        setLastErrMsg("X509_STORE_CTX_init failed!\n");
+        setLastErrMsg("X509_STORE_CTX_init failed!");
         return -1;
     }
     n = X509_STORE_CTX_get1_issuer(&pXissuer, pXstore_ctx, pCert);
     X509_STORE_CTX_free(pXstore_ctx);
     if ((n == -1) || (n == 0))
     {
-        setLastErrMsg("X509_STORE_CTX_get1_issuer failed!\n");
+        setLastErrMsg("X509_STORE_CTX_get1_issuer failed!");
         return -1;
     }
     m_pCertId = OCSP_cert_to_id(NULL, pCert, pXissuer);
