@@ -57,10 +57,10 @@ void VMemBuf::setMaxAnonMapSize(int sz)
     ls_atomic_spin_unlock(&s_LockAnonPool);
 }
 
-int VMemBuf::lowOnAnonMem() 
-{   
+int VMemBuf::lowOnAnonMem()
+{
     ls_atomic_spin_lock(&s_LockAnonPool);
-    int mem = s_iMaxAnonMapBlocks - s_iCurAnonMapBlocks < s_iMaxAnonMapBlocks / 4; 
+    int mem = s_iMaxAnonMapBlocks - s_iCurAnonMapBlocks < s_iMaxAnonMapBlocks / 4;
     ls_atomic_spin_unlock(&s_LockAnonPool);
     return mem;
 }
@@ -167,7 +167,7 @@ void VMemBuf::rewindWOff(off_t rewind)
         {
             rewind -= m_pCurWPos - (*m_pCurWBlock)->getBuf();
             m_pCurWPos = (*m_pCurWBlock)->getBuf();
-            
+
             if ( m_pCurWBlock > m_bufList.begin())
             {
                 m_curWBlkPos -= (*m_pCurWBlock)->getBlockSize();
@@ -187,7 +187,7 @@ void VMemBuf::rewindWOff(off_t rewind)
             }
             else
                 return;
-            
+
         }
         else
         {
@@ -462,9 +462,9 @@ int VMemBuf::appendBlock(BlockBuf *pBlock)
     if (m_bufList.full())
     {
         BlockBuf **pOld = m_bufList.begin();
-        
+
         m_bufList.push_back(pBlock);
-        
+
         if (m_pCurWBlock)
             m_pCurWBlock = m_pCurWBlock - pOld + m_bufList.begin();
         if (m_pCurRBlock)
@@ -577,7 +577,7 @@ void VMemBuf::rewindReadWriteBuf()
 {
     if (m_bufList.empty())
         return ;
-    
+
 #ifdef _RELEASE_MMAP
     if (m_iType == VMBUF_FILE_MAP)
     {
@@ -642,7 +642,7 @@ void VMemBuf::rewindReadBuf()
 {
     if (m_bufList.empty())
         return ;
-    
+
 #ifdef _RELEASE_MMAP
     if (m_iType == VMBUF_FILE_MAP)
     {
@@ -693,7 +693,7 @@ int VMemBuf::mapNextWBlock()
 #ifdef _RELEASE_MMAP
         void *pRelease = NULL;
         int size = 0;
-        
+
         if (m_iType == VMBUF_FILE_MAP)
         {
             if (!(*(m_pCurWBlock + 1))->getBuf())
@@ -812,12 +812,10 @@ char *VMemBuf::getReadBuffer(size_t &size)
     if (m_curRBlkPos == m_curWBlkPos)
     {
         size = m_pCurWPos - m_pCurRPos;
-        assert(size <= 8192);
     }
     else
     {
         size = (*m_pCurRBlock)->getBufEnd() - m_pCurRPos;
-        assert(size <= 8192);
     }
     ls_atomic_spin_unlock(&m_lock);
     return m_pCurRPos;
@@ -866,7 +864,7 @@ int VMemBuf::mapNextRBlock()
 #ifdef _RELEASE_MMAP
         void *pRelease = NULL;
         int size = 0;
-        
+
         if (m_iType == VMBUF_FILE_MAP && m_pCurRBlock != m_pCurWBlock)
         {
             pRelease = (*m_pCurRBlock)->getBuf();
@@ -1222,7 +1220,7 @@ int VMemBuf::empty()
         ret = 0;
     else if (!m_pCurWBlock)
         ret = 1;
-    else 
+    else
         ret = (m_pCurRPos >= m_pCurWPos);
     ls_atomic_spin_unlock(&m_lock);
     return(ret);
